@@ -71,9 +71,11 @@ struct _GLogHandler
 
 
 /* --- prototypes --- */
+#ifndef HAVE_C99_VSNPRINTF
 static gsize printf_string_upper_bound (const gchar *format,
 					gboolean     may_warn,
 					va_list      args);
+#endif /* !HAVE_C99_VSNPRINTF */
 
 
 /* --- variables --- */
@@ -860,6 +862,8 @@ g_printerr (const gchar *format,
 #  define MB_LEN_MAX 8
 #endif
 
+#ifndef HAVE_C99_VSNPRINTF
+
 typedef struct
 {
   guint min_width;
@@ -1168,13 +1172,16 @@ printf_string_upper_bound (const gchar *format,
   return len;
 }
 
+#endif /* !HAVE_C99_VSNPRINTF */
+
+
 gsize
 g_printf_string_upper_bound (const gchar *format,
 			     va_list      args)
 {
 #if HAVE_C99_VSNPRINTF
   gchar c;
-  return vsnprintf (&c, 1, format, args);
+  return vsnprintf (&c, 1, format, args) + 1;
 #else
   return printf_string_upper_bound (format, TRUE, args);
 #endif
